#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>

#include <iostream>
#include <string>

using namespace std;

static bool execSQL( SQLHDBC hdbc, const wchar_t* sql )
{
	SQLHSTMT hstmt = SQL_NULL_HSTMT;
	SQLRETURN status = SQLAllocHandle( SQL_HANDLE_STMT, hdbc, &hstmt );
	status = SQLPrepareW( hstmt, const_cast< wchar_t* >( sql ), SQL_NTS );
	status = SQLExecute( hstmt );
	bool ok = SQL_SUCCEEDED( status ) || status == SQL_NO_DATA;
	if ( !ok )
	{
		wcerr << L"Unable to execute the SQL statement:" << endl << sql << endl;
		SQLRETURN rc;
		SQLWCHAR state[6], msg[4096];
		SQLINTEGER native;
		SQLSMALLINT msgLen;
		for ( SQLSMALLINT i = 1; ( rc = SQLGetDiagRecW( SQL_HANDLE_STMT, hstmt, i, state, &native, msg, 4096, &msgLen ) ) != SQL_NO_DATA; ++i )
			if ( rc == SQL_SUCCESS )
				wcerr << L"[" << i << L"] " << state << L"(" << native << L"): " << msg << endl;
		wcerr << endl;
	}
	status = SQLFreeHandle( SQL_HANDLE_STMT, hstmt );
	return ok;
}

int main( int argc, char** argv )
{
	SQLHANDLE henv = SQL_NULL_HENV;
	SQLRETURN status = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv );
	status = SQLSetEnvAttr( henv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER)SQL_OV_ODBC3, 0 );
	SQLHANDLE hdbc = SQL_NULL_HDBC;
	status = SQLAllocHandle( SQL_HANDLE_DBC, henv, &hdbc );
	status = SQLSetConnectAttrW( hdbc, SQL_LOGIN_TIMEOUT, (SQLPOINTER)5, 0 );
	wchar_t* connectionString = L"DRIVER={PostgreSQL UNICODE};SERVER=localhost;PORT=5432;"
		L"UID=arguser;PWD=CSMCnm42!!;DATABASE=arg;ByteaAsLongVarBinary=1;"
		L"UseDeclareFetch=1;";
	wchar_t outbuf[2048];
	SQLSMALLINT outlen = 0;
	status = SQLDriverConnectW( hdbc, NULL, connectionString, wcslen( connectionString ),
		outbuf, 2048, &outlen, SQL_DRIVER_NOPROMPT );

	execSQL( hdbc, L"DROP TABLE IF EXISTS site;" );
	execSQL( hdbc, L"CREATE TABLE site (id SERIAL PRIMARY KEY, name VARCHAR(64), image_old TEXT, image BYTEA);" );
	execSQL( hdbc, L"UPDATE site SET image=image_old;" );
	execSQL( hdbc, L"SELECT id FROM site;" );
	execSQL( hdbc, L"SELECT name FROM site;" );

	SQLDisconnect( hdbc );
	SQLFreeHandle( SQL_HANDLE_DBC, hdbc );
	SQLFreeHandle( SQL_HANDLE_ENV, henv );
	return 0;
}
